@@ -1,7 +1,8 @@
 # coding: utf-8
-from __future__ import unicode_literals
+from __future__ import unicode_literals, division
 import re
 import logging
+import math
 
 import dateutil.parser
 import six
@@ -13,12 +14,18 @@ from .exceptions import InvalidRequestError, APIError
 
 logger = logging.getLogger("pyuploadcare")
 
+
+RE_UUID = '[a-z0-9]{8}-(?:[a-z0-9]{4}-){3}[a-z0-9]{12}'
+RE_EFFECTS = '(?:[^/]+/)+'  # -/resize/(200x300/)*
 UUID_WITH_EFFECTS_REGEX = re.compile('''
-    (?P<uuid>[a-z0-9]{8}-(?:[a-z0-9]{4}-){3}[a-z0-9]{12})
-    (
-        /-/(?P<effects>.*)
+    /?
+    (?P<uuid>{uuid})  # required
+    (?:
+        /
+        (?:-/(?P<effects>{effects}))?
+        ([^/]*)  # filename
     )?
-''', re.VERBOSE)
+$'''.format(uuid=RE_UUID, effects=RE_EFFECTS), re.VERBOSE)
 
 
 class File(object):
@@ -59,6 +66,13 @@ class File(object):
     def _api_storage_uri(self):
         return 'files/{0}/storage/'.format(self.uuid)
 
+    def cdn_path(self, effects=None):
+        ptn = '{uuid}/-/{effects}' if effects else '{uuid}/'
+        return ptn.format(
+            uuid=self.uuid,
+            effects=effects
+        )
+
     @property
     def cdn_url(self):
         """Returns file's CDN url.
@@ -76,17 +90,8 @@ class File(object):
             http://www.ucarecdn.com/a771f854-c2cb-408a-8c36-71af77811f3b/-/effect/flip/-/effect/mirror/
 
         """
-        if self.default_effects:
-            return '{cdn_base}{uuid}/-/{effects}'.format(
-                cdn_base=conf.cdn_base,
-                uuid=self.uuid,
-                effects=self.default_effects
-            )
-        else:
-            return '{cdn_base}{uuid}/'.format(
-                cdn_base=conf.cdn_base,
-                uuid=self.uuid
-            )
+        return '{cdn_base}{path}'.format(cdn_base=conf.cdn_base,
+                                         path=self.cdn_path(self.default_effects))
 
     def info(self):
         """Returns all available file information as ``dict``.
@@ -203,6 +208,25 @@ class File(object):
         """
         self._info_cache = rest_request('PUT', self._api_storage_uri)
 
+    def copy(self, effects=None, target=None):
+        """Creates File copy
+
+        If ``target`` is ``None``, copy file to Uploadcare storage otherwise
+        copy to target associated with project.
+        Add ``effects`` to ``self.default_effects`` if any.
+        """
+        effects = effects or ''
+        if self.default_effects is not None:
+            effects = '{head}-/{tail}'.format(head=self.default_effects,
+                                              tail=effects)
+        data = {
+            'source': self.cdn_path(effects)
+        }
+        if target is not None:
+            data['target'] = target
+
+        return rest_request('POST', 'files/', data=data)
+
     def delete(self):
         """Deletes file by requesting Uploadcare API."""
         self._info_cache = rest_request('DELETE', self._api_uri)
@@ -321,6 +345,121 @@ class File(object):
                 return File(self.info()['uuid'])
 
 
+class FileList(object):
+    """List of File resources.
+
+    This class provides iteration over all uploaded files. You can specify:
+
+    - ``offset`` -- an offset into the list of returned items;
+    - ``count`` -- a limit on the number of objects to be returned;
+    - ``stored`` -- ``True`` to include only removed files,
+      ``False`` to exclude;
+    - ``removed`` -- ``True`` to include only stored files,
+      ``False`` to exclude.
+
+    Usage example::
+
+        >>> files_from_second_to_end = FileList(offset=1)
+        >>> for file_ in files_from_second_to_end:
+        >>>     print file_
+
+    """
+
+    def __init__(self, offset=0, count=None, stored=None, removed=None):
+        if offset < 0:
+            raise InvalidRequestError(
+                'offset has to be greater than or equal to zero'
+            )
+        self.offset = offset
+
+        if count is not None and count < 0:
+            raise InvalidRequestError(
+                'count has to be greater than or equal to zero'
+            )
+        self.count = count
+
+        self.stored = stored
+        self.removed = removed
+
+    def __iter__(self):
+        return FileList.FileListIterator(self.offset, self.count,
+                                         self.stored, self.removed)
+
+    @classmethod
+    def retrieve(cls, page, limit=20, stored=None, removed=None):
+        """Returns list of files' raw information by requesting Uploadcare API.
+        """
+        if page < 1:
+            raise InvalidRequestError(
+                'page has to be greater than or equal to one'
+            )
+
+        url = 'files/?page={page}&limit={limit}'.format(page=page, limit=limit)
+        if stored is not None:
+            url += '&stored={stored}'.format(
+                stored='true' if stored else 'false')
+        if removed is not None:
+            url += '&removed={removed}'.format(
+                removed='true' if removed else 'false')
+        return rest_request('GET', url)
+
+    class FileListIterator(object):
+        """Iterator that yields ``File`` instances while API pages are found.
+
+        It caches API result for particular page between yields.
+
+        """
+
+        _count_per_request = 20
+
+        def __init__(self, offset, count=None, stored=None, removed=None):
+            # ``+1`` is a zero numbering correction.
+            self._page = int(math.ceil(
+                (offset + 1) / self._count_per_request
+            ))
+            self._position_in_page = offset % self._count_per_request
+
+            self._count = count
+            self._stored = stored
+            self._removed = removed
+
+            self._count_of_constructed_files = 0
+            self._result_cache = None
+
+        def next(self):
+            if (self._count is not None and
+                self._count == self._count_of_constructed_files):
+                raise StopIteration
+
+            if self._result_cache is None or self._result_cache['page'] != self._page:
+                try:
+                    self._result_cache = FileList.retrieve(
+                        page=self._page,
+                        limit=self._count_per_request,
+                        stored=self._stored,
+                        removed=self._removed
+                    )
+                except InvalidRequestError:
+                    raise StopIteration
+            try:
+                file_info = self._result_cache['results'][self._position_in_page]
+            except IndexError:
+                raise StopIteration
+
+            # ``-1`` is a zero numbering correction.
+            if self._position_in_page < self._count_per_request - 1:
+                self._position_in_page += 1
+            else:
+                self._page += 1
+                self._position_in_page = 0
+
+            self._count_of_constructed_files += 1
+
+            return File.construct_from(file_info)
+
+        __next__ = next
+
+
 GROUP_ID_REGEX = re.compile('''
     (?P<group_id>
         [a-z0-9]{8}-(?:[a-z0-9]{4}-){3}[a-z0-9]{12}
